'' ILI9325 driver using the Touchblade161 design for faster ram to display transfers
'' Average Joe and James Moxham 2012

CON


  _clkmode      = xtal1 + pll16x                        ' use crystal x 16
  _xinfreq      = 5_000_000

  _1ms    = 1_000_000 /     1_000                         ' Divisor for 1 ms   

' pins
'  25,26,27   138 one of 8 decoded, active low
' 0=000 = SPI touchscreen input, keyboard, TV VGA enabled
' 1=001 = Load counter
' 2=010 = Memory transfer - P0-P15=bus, P19=161_clk, P17 = ILI_WR, P18=ILI_RS, P16= Ram_RD, P20=Ram_WR,P21-23 = SD card
' 3=011 = ILI Reset
' 4=100 = Digital output (8 bits to a 574 latch)
' 5=101 = spare
' 6=110 = spare
' 7=111 = startup (all above deselected)

' SD pins
  _SD_DO = 21
  _SD_CLK = 22
  _SD_DI = 23
  _SD_CS = 24                                          
  _SD_WP = -1                                           ' -1 ifnot installed.
  _SD_CD = -1                                           ' -1 ifnot installed.
  _RTC_DAT = -1                                         ' -1 ifnot installed. For kye's sd driver
  _RTC_CLK = -1                                         ' -1 ifnot installed.

  'RamText   = 76800
  FontTable = 76800                                    ' address for ram chip (2 bytes per address)

OBJ

 fat:           "SD3.01_FATEngine.spin"                 ' thanks to Kye

VAR
   long orientation
   long curx, cury
   long clkcycles               ' for the delay routine
   word ScreenWidth             ' either 240 in portrait or 320 in landscape
   word ScreenHeight            ' 320 in portrait or 240 in landscape
   byte LineOfText1[20]         ' general purpose string buffer
   byte LineOfText2[20]         ' general purpose string buffer
   byte FontHeight              ' size of the current loaded font (pixels = fontsize *.75)
   word BackFontColor           ' the background color in RRRRRGGG GGGBBBBB format

   
PUB Main
  curx :=0
  cury :=0
  dira := 0                                             ' pins all tristated
  SetOrientation(true)                                  '  in portrait (true) or landscape mode (false) 
  start_ram                                             ' start the external ram driver in a cog
  Select138(2)                                          ' select 138 and set control pins
  Start_ILI9325                                         ' start the display
  Clearscreen(0)                                        ' clear screen to black
  Propfont_string(string("SD"))                         ' string to send, uses internal prop font so can see something if the SD fails to mount   
  ifnot( fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))   
    reboot
  fat.mountPartition(0)                               ' mount the sd card
  Bootmessage
  LoadPicture
  DisplayPicture
  

PUB Bootmessage
    ILIloadfont(string("JDLCD.ifn"))                     ' LED dot matrix type font green on black         
    ILISetcursor(0,0)                                   ' set cursor to this location
    ClearFontBackground(0,0,screenwidth,FontHeight)                 ' clear an area the same color as the font background       
    ILIprintstring(string(">Booting"))  

PUB ClearFontBackground(x1,y1,x2,y2) | i,size,remainder ' clears an area of the screen to the current font background color
    size := (x2-x1+1) * (y2 - y1 +1)                    ' number of pixels
    remainder := size & 255                             ' if not a whole number of 256 pixels
    repeat i from 0 to 255                              ' move the background font colour to the buffer
      rambuffer[i] := BackFontColor
    Draw(x1,y1,x2,y2)                                   ' set up the area of the screen to draw in
    repeat size >> 8                                    ' 256 pixel blocks
      docmd("V",@rambuffer,0,256)                
    if remainder <> 0  
      docmd("V",@rambuffer,0,remainder)                 ' do the remainder

PUB ILIChar(ascii,x,y) | jump,size,width,height,ramaddress,xoffset,yoffset,xadvance   ' read out a character from a .ifn file stored in ram
  ' pass Fonttable = global
  ' moves to next line if off the end
    jump := ReadRamLong2(fonttable,(ascii << 2) + 256)' jump location = ascii *4 plus fonttable + 256
    'jump += fonttable                                     ' add the fonttable location
    size := ReadRamLong2(fonttable,jump)>>1     ' size precalculated = width x height x 2 so divide by 2
    xadvance :=  ReadRamLong2(fonttable,jump+20) ' ' amount to move to next character
    if size > 0 and ascii <> 32                                           ' no need to print anything if size is zero or read more font data
      width :=   ReadRamLong2(fonttable,jump+4)     ' width
      height :=  ReadRamLong2(fonttable,jump+8) ' height
      xoffset := ReadRamLong2(fonttable,jump+12) ' xoffset to move
      yoffset := ReadRamLong2(fonttable,jump+16) ' yoffset to move
      if (x+width -1 + xoffset) > screenwidth
        ILIcrlf ' do a new line if it won't fit
        x :=curx
        y :=cury
      Draw(x+xoffset,y+yoffset,x+width-1+xoffset,y+height-1+yoffset)      ' draw on screen
      RamToDisplay(fonttable+((jump+32)>>1),size) ' move bytes from ram out to the display
    return x + xadvance

PUB ILIcrlf
    curx := 0
    cury += FontHeight
    if cury > screenheight
      ClearFontBackground(0,0,screenwidth,screenheight) 
      cury := 0                 ' if off the bottom of the screen then clear screen start at the top (VT00 does scrolling)
    ' don't clear background here, do in calling routine
    'ClearFontBackground(0,cury,screenwidth,cury + FontHeight) ' clear background line to the background color ready to draw          


PUB ILIPrintstring(stringptr)                 'print at curx,cury
'' Print a zero-terminated string
  repeat strsize(stringptr)
    curx := ILIChar(byte[stringptr++],curx,cury)


PUB ILISetCursor(x,y)            
    curx := x
    cury := y

PUB ReadRamLong(Address)                                ' read a long, stored LSB first, address is as per hexedit address
    docmd("T",@result,Address>>1,2)                     ' get two words and read in one after the other
    return result

PUB ReadRamLong2(WordStart,Bytevalue)                   ' easier calculating
    docmd("T",@result,Wordstart+Bytevalue>>1,2)
    return result

PUB ReadRamByte(Address)                                ' read as if a continous byte file
    docmd("T",@result,Address>>1,1)                     ' read one word
    address &= 1                                        ' is it even or odd
    if address == 1
      result := result >> 8                             ' odd so shift
    else
      result &= $FF                                     ' even so mask
    return result    

PUB ILILoadFont(stringptr) | filesize,i                ' read a .ifn file premade in angelcode bmfont and then the vb.net program
'    ' pass file name in stringptr
    i := fonttable                                      ' store at the constant 'fonttable' location, 2 bytes per address
    fat.openfile(stringptr,"R")                         ' open an image. Size is in the first long and has been rounded up to the next 256 byte block
    filesize := fat.filesize                            ' read the file size. This file has been rounded up to the next 256 byte block
    repeat (filesize >> 8)                              ' read in this number of 256 byte blocks
      fat.readdata(@rambuffer,256)                      ' get 256 bytes from sd card
      HubToRam(i,128)                                   ' 128 words = 256 bytes 
      i += 128                                          ' counter for ram location
    fat.closefile
    FontHeight := ReadRamByte((fonttable<<1)+251)    ' extract font size
    ' extract the background color so can draw the background behind the text
    repeat i from 0 to 2
      sdbuffer[i] := readrambyte(fonttable<<1 + 35+i)   ' read the 3 RGB bytes
    docmd("E",@sdbuffer,@buffer2,1)             ' convert to 2 byte .ili format, len is number of pixels        
    BackFontColor := word[@buffer2][0]
    result := filesize

PUB LoadPicture | i
    i := 0
    fat.openfile(string("lunch.ili"),"R")
    repeat 300
      fat.readdata(@rambuffer,512)                      ' get 512 bytes from sd card
      HubToRam(i,256)                                   ' 256 words = 512 bytes 
      i += 256                                         ' counter for ram location
    fat.closefile
PUB DisplayPicture
    ' display picture
    Draw(0,0,239,319)  ' dump out 76800 words to the display = a full screen
    RamToDisplay(0,76800)
        

PUB HubToRam(RamAddress,Number)                         ' send pixels from hub to ram
   docmd("S",@rambuffer,RamAddress,Number)

PUB RamToHub(RamAddress,Number)                         ' send pixels from ram to hub
   docmd("T",@rambuffer,RamAddress,Number)

PUB RamToDisplay(RamAddress,Number)                     ' send pixels from ram to display
   docmd("U",0,RamAddress,Number)

Pub HubToDisplay(HubAddress,Number)                     ' send pixels from hub to display
   docmd("V",HubAddress,0,Number)   

PUB Clearscreen(color) 
    wordfill(@rambuffer,color,256)
    HubToRam(0,256)                                     ' send 256 pixels from hub to ram
    Draw(0,0,screenwidth,screenheight)                           ' set up the area of the screen to draw in   
    repeat 300
      RamToDisplay(0,256)                       

PUB SetOrientation(e) ' change orientation true = portrait. Changes global variable 'orientation' and also screen width and height
   orientation := e
   ChangeOrientation(orientation)
   if orientation
     screenwidth :=239
     screenheight :=319
   else
     screenwidth :=319
     screenheight :=239

PUB ChangeOrientation(n) ' pass true = portrait or false = landscape, changes global variable orientation in this object
    orientation := n
    if orientation
      ILIcmd($0001,$0100)                        ' set SS and SM bit 0001 0100   portrait
      ILIcmd($0003,$1030)                        ' set GRAM write direction and BGR=1. $0003 $1030  
    else
      ILIcmd($0001,$0000)                        ' set SS and SM bit 0001 0000   landscape   
      ILIcmd($0003,$1038)                        ' landscape  $1028 = original but 1038 is correct - not mirror image

PUB Select138(n) ' pass n=0 to 7
  ' come to this with possibly all pins tristated so need to make P16-P20 high before changing the 138 value
   OUTA |= %00001110_00011111_00000000_00000000         ' all control pins high and P25-P27 high 
   DIRA |= %00001110_00011111_11111111_11111111         ' outputs enabled
   n := n<<25                                           ' shift left so pins 25,26,27
   n |= %11110001_11111111_11111111_11111111            ' set other pins high
   OUTA := n

PUB SDMessage  ' use propeller font in the rom for bootup messages when debugging sd cards
    Propfont_string(string("Mount SD")) ' string to send

PUB Propfont_string(stringptr)                          'print at curx,cury
  repeat strsize(stringptr)
    Propfont_out(byte[stringptr++])
  crlf

PUB crlf
    curx := 0
    cury += 32                                          ' new line at end of string
    if cury >319                                       ' bottom of screen so new screen
      curx:=0
      cury:=0

PUB Propfont_out(ascii) | address,pixels
    Draw(curx,cury,curx+15,cury+31)                 ' location to start drawing 
    address := $8000 + (ascii >> 1) << 7                ' get rom address
    repeat 32                                           ' 32 rows per character, split in two parts
      pixels := long[address]                           ' get rom font data
      pixels := pixels >> (ascii & 1)                    ' shift for odd characters
      repeat 16 ' 16 columns
        if pixels & 1
          Pixel(%00000111_11100000)                     ' foreground color RRRRRGGG_GGGBBBBB
        else
          Pixel(%00000000_00000000)                     ' background color 
        pixels := pixels >> 2                           ' alternate pixels interleaved so shift 2
      address += 4 
    curx +=16
    if curx >239                                        ' new line
      crlf

PUB Start_ILI9325 ' pass orientation true = portrait, false = landscape
    ILI_Reset_High
    pause1ms(5)
    ILI_Reset_Low
    pause1ms(5)
    ILI_Reset_High
    ILI_CS_High
    ILI_RD_High
    ILI_WR_High
    pause1ms(5)
    ILI_CS_Low
          ' ************* Start Initial Sequence **********
    ILIcmd($00E5,$78F0)                          ' set SRAM internal timing
    ILIcmd($0001,$0100)                        ' set SS and SM bit 0001 0100   portrait
    ILIcmd($0002,$0700)                          ' set 1 line inversion 
    ILIcmd($0003,$1030)                        ' set GRAM write direction and BGR=1. $0003 $1030
    ILIcmd($0004,$0000)                          ' Resize register 
    ILIcmd($0008,$0207)                          ' set the back porch and front porch 
    ILIcmd($0009,$0000)                          ' set non-display area refresh cycle ISC[3:0]  
    ILIcmd($000A,$0000)                          ' FMARK function  
    ILIcmd($000C,$0000)                          ' RGB interface setting   
    ILIcmd($000D,$0000)                          ' Frame marker Position   
    ILIcmd($000F,$0000)                          ' RGB interface polarity  
'        *************Power On sequence ****************//
    ILIcmd($0010,$0000)                         ' SAP, BT[3:0], AP, DSTB, SLP, STB 
    ILIcmd($0011,$0007)                         ' DC1[2:0], DC0[2:0], VC[2:0] 
    ILIcmd($0012,$0000)                         ' VREG1OUT voltage   
    ILIcmd($0013,$0000)                         ' VDV[4:0] for VCOM amplitude  
    ILIcmd($0007,$0001)   
    pause1ms(50)                                        ' Dis-charge capacitor power voltage
    ILIcmd($0010,$1090)                         ' 1490//SAP, BT[3:0], AP, DSTB, SLP, STB   
    ILIcmd($0011,$0227)                         ' DC1[2:0], DC0[2:0], VC[2:0] 
    pause1ms(50)                                        ' delay
    ILIcmd($0012,$001F)                         ' 001C// Internal reference voltage= Vci;
    pause1ms(50)                                        ' delay   
    ILIcmd($0013,$1500)                         ' $1000//1400   Set VDV[4:0] for VCOM amplitude  1A00  
    ILIcmd($0029,$0027)                         ' $0012 //001a  Set VCM[5:0] for VCOMH  //$0025  0034  
    ILIcmd($002B,$000D)                         ' Set Frame Rate   000C       
    pause1ms(50)                                        ' delay   
    ILIcmd($0020,$0000)                         ' GRAM horizontal Address   
    ILIcmd($0021,$0000)                         ' GRAM Vertical Address    
'         ----------- Adjust the Gamma Curve ----------//
    ILIcmd($0030,$0000) 
    ILIcmd($0031,$0707)  
    ILIcmd($0032,$0307)  
    ILIcmd($0035,$0200)  
    ILIcmd($0036,$0008)  
    ILIcmd($0037,$0004)  
    ILIcmd($0038,$0000)  
    ILIcmd($0039,$0707)  
    ILIcmd($003C,$0002)  
    ILIcmd($003D,$1D04)  
'        ------------------ Set GRAM area ---------------//
    ILIcmd($0050,$0000)  ' Horizontal GRAM Start Address
    ILIcmd($0051,$00EF)  ' Horizontal GRAM End Address       
    ILIcmd($0052,$0000) ' Vertical GRAM Start Address    
    ILIcmd($0053,$013F) ' Vertical GRAM Start Address       
    ILIcmd($0060,$A700) ' Gate Scan Line     
    ILIcmd($0061,$0001) ' NDL,VLE, REV    
    ILIcmd($006A,$0000) ' set scrolling line      
'        -------------- Partial Display Control ---------/
    ILIcmd($0080,$0000)
    ILIcmd($0081,$0000)
    ILIcmd($0082,$0000)
    ILIcmd($0083,$0000)
    ILIcmd($0084,$0000)
    ILIcmd($0085,$0000)
'        //-------------- Panel Control -------------------//
    ILIcmd($0090,$0010)
    ILIcmd($0092,$0600)
    ILIcmd($0007,$0133)  ' 262K color and display ON
    ChangeOrientation(true)     ' default to portrait

PUB Draw(x1, y1, x2, y2)       ' sets the pixel to x1,y1 and then fills the next (x2-x1)*(y2-y1) pixels
    select138(2)                                        ' set 138
    ifnot orientation                                   ' landscape mode so swap x and y
      result :=x1                                       ' swap x1 and y1
      x1 := y1
      y1 := result
      result := x2                                      ' swap x2 and y2
      x2 :=y2
      y2 := result
    ILIcmd($0050,x1)
    ILIcmd($0052,y1)
    ILIcmd($0051,x2)
    ILIcmd($0053,y2)
    ILIcmd($0020,x1) 
    ILIcmd($0021,y1)
    Lcd_Write_Com($0022)
  


PUB Pixel(pixelcolor)               ' send out a pixel
    Lcd_Write_Data(pixelcolor)

PUB pause1ms(period)

'' Pause execution for period (in units of 1 ms).

  clkcycles := ((clkfreq / _1ms * period) - 4296) #> 381     ' Calculate 1 ms time unit
  waitcnt(clkcycles + cnt)                                   ' Wait for designated time

PUB hex(value, digits)   ' uses internal prop font and slow pixel drawing (for very basic debugging without an SD font)
  select138(2)                               ' reselect 138 for display output for debugging  
'' Print a hexadecimal number
  propfont_out("O")
  propfont_out("x")
  value <<= (8 - digits) << 2
  repeat digits
    propfont_out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))
  crlf
                   

PRI ILIcmd(c,d)  ' instruction in one method
    Lcd_Write_Com(c) ' send out a word
    Lcd_Write_Data(d)

PRI Lcd_Write_Com(ILIlong)
    ILI_RS_low
    LCD_Writ_Bus(ILIlong)

PRI Lcd_Write_Data(ILIlong)
    ILI_RS_High
    LCD_Writ_Bus(ILIlong)    

PRI LCD_Writ_Bus(ILILong)
    'ILILong &= %00000000_00000000_11111111_11111111    ' mask to a word. Not needed if care taken always to send a word value
    OUTA &= %11111111_11111111_00000000_00000000        ' set P0-P15 to zero ready to OR
    OUTA |= ILILong                                     ' merge with the word to output
    ILI_WR_Low                                          ' send out the data
    ILI_WR_High

PRI ILI_RS_Low
   OUTA &= %11111111_11111011_11111111_11111111         ' P18 low

PRI ILI_RS_High   
   OUTA |= %00000000_00000100_00000000_00000000         ' P18 high

PRI ILI_RD_Low                                          ' pulled high in hardware, not used  
PRI ILI_RD_High                                           

PRI ILI_WR_Low
   OUTA &= %11111111_11111101_11111111_11111111         ' p17 low

PRI ILI_WR_High
   OUTA |= %00000000_00000010_00000000_00000000         ' p17 high

PRI ILI_CS_Low                                          ' not used as setting 138 to %010 does this
PRI ILI_CS_High   

PRI ILI_RESET_Low                                       ' reset low
   Select138(3)                                         ' reset low

PRI ILI_RESET_High
   Select138(2)                                         ' 138 back to memselect  
  

DAT
sdbuffer        byte    $0[512]  ' 512 byte buffer for sd card interface 
rambuffer       word    $0[256]  ' 256 words = 512 bytes
buffer2         byte    $0[512]  ' 512 byte general purpose hub buffer

PUB start_ram : err_
' Initialise the Drac Ram driver. No actual changes to ram as the read/write routines handle this
  command := "I"
  cog := 1 + cognew(@tbp2_start, @command)
  if cog == 0
    err_ := $FF                 ' error = no cog
  else
    repeat while command        ' driver cog sets =0 when done
    err_ := errx                ' driver cog sets =0 if no error, else xx = error code

PUB stop_ram
   if cog
      cogstop(cog~ - 1)      

PUB DoCmd(command_, hub_address, ram_address, block_length) : err_
' Do the command: A-Z (I is reserved for Initialise)
  dira := 0                     ' tristate all pins with the spin dira
  hubaddrs := hub_address       ' hub address start
  ramaddrs := ram_address       ' ram address start
  blocklen := block_length      ' block length
  command  := command_          ' must be last !!
' Wait for command to complete and get status
  repeat while command          ' driver cog sets =0 when done
  err_ := errx                  ' driver cog sets =0 if no error, else xx = error code

CON
''Dracblade driver for talking to a ram chip via three latches
'' Modified code from Cluso's triblade
'' also includes commands to move blocks of data to the ILI9325 touchscreen display
' DoCmd(command_, hub_address, ram_address, block_length)
' I - initialise     
' S - Move data from hub to ram
' T - Move data from ram to hub
' U - Move data from ram to display

VAR

' communication params(5) between cog driver code - only "command" and "errx" are modified by the driver
   long  command, hubaddrs, ramaddrs, blocklen, errx, cog ' rendezvous between spin and assembly (can be used cog to cog)
'        command  = A to Z etc =0 when operation completed by cog
'        hubaddrs = hub address for data buffer
'        ramaddrs = ram address for data
'        blocklen = ram buffer length for data transfer
'        errx     = returns =0 (false=good), else <>0 (true & error code)
'        cog      = cog no of driver (set by spin start routine)
   

DAT
'' +-----------------------------------------------------------------------------------------------+
'' | Touchblade 161 Ram Driver (with grateful acknowlegements to Cluso and Average Joe)            |
'' +-----------------------------------------------------------------------------------------------+
                        org     0
tbp2_start    ' setup the pointers to the hub command interface (saves execution time later
                                      '  +-- These instructions are overwritten as variables after start
comptr                  mov     comptr, par     ' -|  hub pointer to command                
hubptr                  mov     hubptr, par     '  |  hub pointer to hub address            
ramptr                  add     hubptr, #4      '  |  hub pointer to ram address            
lenptr                  mov     ramptr, par     '  |  hub pointer to length                 
errptr                  add     ramptr, #8      '  |  hub pointer to error status           
cmd                     mov     lenptr, par     '  |  command  I/R/W/G/P/Q                  
hubaddr                 add     lenptr, #12     '  |  hub address                           
ramaddr                 mov     errptr, par     '  |  ram address                           
len                     add     errptr, #16     '  |  length                                
err                     nop                     ' -+  error status returned (=0=false=good) 


' Initialise hardware tristates everything and read/write set the pins
init                    mov     err, #0                  ' reset err=false=good
                        mov     dira,zero                ' tristate the pins with the cog dira

done                    wrlong  err, errptr             ' status  =0=false=good, else error x
                        wrlong  zero, comptr            ' command =0 (done)
' wait for a command (pause short time to reduce power)
pause
'                        mov     ctr, delay      wz      ' if =0 no pause
'              if_nz     add     ctr, cnt
'              if_nz     waitcnt ctr, #0                 ' wait for a short time (reduces power)
                        rdlong  cmd, comptr     wz      ' command ?
              if_z      jmp     #pause                  ' not yet
' decode command
                        cmp     cmd, #"S"       wz      ' hub to ram
              if_z      jmp     #pasmhubtoram           
                        cmp     cmd, #"T"       wz      ' ram to hub
              if_z      jmp     #pasmramtohub
                        cmp     cmd, #"U"       wz      ' ram to display
              if_z      jmp     #pasmramtodisplay
                        cmp     cmd, #"V"       wz      ' hub to display
              if_z      jmp     #pasmhubtodisplay           
                        cmp     cmd, #"E"       wz      ' convert 3 byte .raw format to 2 byte .ili format - hub to hub
              if_z      jmp     #rawtoiliformat
                        cmp     cmd, #"I"       wz      ' init
              if_z      jmp     #init     
                        mov     err, cmd                ' error = cmd (unknown command)
                        jmp     #done
                        
' ----------------- common routines -------------------------------------

get_values              rdlong  hubaddr, hubptr         ' get hub address
                        rdlong  ramaddr, ramptr         ' get ram address
                        rdlong  len, lenptr             ' get length
                        mov     err, #5                 ' err=5
get_values_ret          ret

                    ' ??come to this with possibly all pins tristated so need to make P16-P20 high before changing the 138 value 
set138                  shl     pasm_n,#25              ' pass n =0 to 7
                        or      dira,maskP0P20P25P27    ' make P25-P27 outputs as well as P0 to P20
                        andn    outa,mask138            ' make these three pins low
                        or      outa,pasm_n             ' set the 138 pins
set138_ret              ret


load161pasm                                             ' uses ramaddr
                        mov     pasm_n,#7
                        call    #set138                 ' deselect previous 138 value
                        or      dira,maskP0P20          ' %00000000_00011111_11111111_11111111         ' P0-P18 enabled for output plus P19,P20 
                        and     outa,maskP18low         ' %00001111_11111000_00000000_00000000         ' preserve previous values but set A0-18 low   
                        or      outa,ramaddr            ' output address to 161 chips
                        andn    outa,maskP19            ' set pin 19 low =  161 clock
                        mov     pasm_n,#1               ' 161 load low
                        call    #set138                 ' set it low
                        or      outa,maskP19            ' set pin 19 high = 161 clock
                        or      outa,maskP16P20         ' %00000000_00011111_00000000_00000000         ' set P16-P20 high prior to changing 138 
                        mov     pasm_n,#2               ' 161 load high and back to mem transfer
                        call    #set138                 ' send out
load161pasm_ret         ret

stop                   jmp     #stop                  ' for debugging



' ------------------ single letter commands  -------------------------------------
' command S
pasmhubtoram            call    #get_values             ' get hubaddr,ramaddr,len
                        call    #load161pasm                ' load the 161 counters with ramaddr
hubtoram_loop           and     outa,maskP16P31         '%11111111_11111111_00000000_00000000       ' clear for output                   
                        rdword  data_16,hubaddr         ' get the word from hub
                        and     data_16,maskP0P15       ' mask to a word only
                        or      outa,data_16            ' send out the byte to P0-P15
                        andn    outa,maskP20            ' set write low
                        add     hubaddr,#2              ' increment by 2 bytes = 1 word. Put this here for small delay while writes
                        or      outa,maskP20            ' write high
                        andn    outa,maskP19            ' clock 161 low
                        or      outa,maskP19            ' clock 161 high
                        djnz    len,#hubtoram_loop      ' loop this many times
                        jmp     #init                   ' tristate pins and listen for commands

' command T
pasmramtohub            call    #get_values             ' get hubaddr,ramaddr,len
                        call    #load161pasm            ' load the 161 counters with ramaddr
                        and     dira,maskP16P27         ' %00001111_11111111_00000000_00000000 set P0-P15 as inputs   
                        andn    outa,maskP16            ' memory /rd low
ramtohub_loop           mov     data_16,ina             ' get the data
                        wrword  data_16,hubaddr         ' move data to hub
                        andn    outa,maskP19            ' clock 161 low
                        or      outa,maskP19            ' clock 161 high
                        add     hubaddr,#2              ' increment the hub address 
                        djnz    len,#ramtohub_loop
                        or      outa,maskP16            ' memory /rd high  
                        or      dira,maskP0P15          ' %00000000_00000000_11111111_11111111 restore P0-P15as outputs
                        jmp     #init                   ' ' tristate pins and listen for commands

' command U
pasmramtodisplay        call    #get_values             ' get hubaddr,ramaddr,len
                        call    #load161pasm            ' load the 161 counters with ramaddr
                        or      outa,maskP18            ' ILI_RS high
                        andn    outa,maskP16            ' memory /rd low 
                        and     dira,maskP16P27         ' disable prop pins %00001111_11111111_00000000_00000000 set P0-P15 as inputs    
ramtodisplay_loop       andn    outa,maskP17            ' ILI write low
                        or      outa,maskP17            ' ILI write high
                        andn    outa,maskP19            ' clock 161 low
                        or      outa,maskP19            ' clock 161 high
                        djnz    len,#ramtodisplay_loop
                        or      outa,maskP16            ' memory /rd high  
                        or      dira,maskP0P15          ' %00000000_00000000_11111111_11111111 restore P0-P15as outputs
                        jmp     #init

' command V
pasmhubtodisplay        call    #get_values             ' get hubaddr,ramaddr,len
                        or      outa,maskP16P20         ' %00000000_00011111_00000000_00000000         ' set P16-P20 high prior to changing 138
                        mov     pasm_n,#7
                        call    #set138                 ' deselect previous 138 value
                        mov     pasm_n,#2               ' mem transfer
                        call    #set138                 ' send out
hubtodisplay_loop       and     outa,maskP16P31         '%11111111_11111111_00000000_00000000       ' clear for output                   
                        rdword  data_16,hubaddr         ' get the word from hub
                        and     data_16,maskP0P15       ' mask to a word only
                        or      outa,data_16            ' send out the byte to P0-P15
                        andn    outa,maskP17            ' ILI write low
                        or      outa,maskP17            ' ILI write high
                        add     hubaddr,#2              ' one word
                        djnz    len,#hubtodisplay_loop
                        jmp     #init

'command E
RawtoILIformat          ' takes a .raw 3 byte RRRRRRRR GGGGGGGG BBBBBBBB and converts to 2 byte RRRRRGGG GGGBBBBB
                        ' pass hubaddr, ramaddr and len
                        ' hubaddr is source location, len is number of pixels
                        ' ramaddr is destination in hub (messy naming) and length is 2/3 of blocklength
                        call    #get_values ' gets hubaddress, ramaddress and len (ignores ramaddress)
rawloop
                        rdbyte red,hubaddr
                        add hubaddr,#1
                        rdbyte green,hubaddr
                        add hubaddr,#1
                        rdbyte blue,hubaddr
                        add hubaddr,#1
                        call #rgbtoili
                        wrbyte ililow,ramaddr
                        add ramaddr,#1
                        wrbyte ilihigh,ramaddr
                        add ramaddr,#1
                        djnz    len,#rawloop            ' loop until done 
                        jmp     #init                   ' set pins to tristate

RGBtoILI                ' pass red,green, blue, returns ililow and ilihigh
                        shr     red,#3                  ' 000RRRRR 
                        shl     red,#3                  ' RRRRR000 
                        shr     green,#2                ' 00GGGGGG
                        mov     ilihigh,green           ' ilihigh = 00GGGGGG
                        shr     ilihigh,#3              ' ilihigh = 00000GGG
                        or      ilihigh,red             ' ilihigh = RRRRRGGG
                        and     green,#%00000111        ' 00000GGG
                        shl     green,#5                ' GGG00000
                        mov     ililow,green            ' ililow = GGG00000
                        shr     blue,#3                 ' blue = 000BBBBB
                        or      ililow,blue             ' ililow = GGGBBBBB
RGBtoILI_ret            ret






' variables
pasm_n                  long    0                                    ' general purpose value
data_16                 long    0                                    ' general purpose value
ililow                  long    0                                    ' low data byte latch5
ilihigh                 long    0                                    ' high data byte latch6
red                     long    0                                    ' red, green blue variables
green                   long    0
blue                    long    0           

' constants
Zero                    long    %00000000_00000000_00000000_00000000 ' used in several places
mask138                 long    %00001110_00000000_00000000_00000000 ' mask for the three 138 pins   
maskP0P20               long    %00000000_00011111_11111111_11111111 ' P0-P18 enabled for output plus P19,P20    
maskP18low              long    %00001111_11111000_00000000_00000000 ' P0-P18 low
maskP16                 long    %00000000_00000001_00000000_00000000 ' pin 16
maskP17                 long    %00000000_00000010_00000000_00000000 ' pin 17
maskP18                 long    %00000000_00000100_00000000_00000000 ' pin 18
maskP19                 long    %00000000_00001000_00000000_00000000 ' pin 19
maskP20                 long    %00000000_00010000_00000000_00000000 ' pin 20
maskP16P31              long    %11111111_11111111_00000000_00000000 ' pin 16 to pin 31
maskP0P20P25P27         long    %00001110_00011111_11111111_11111111  ' enable all pins as outputs except SD pins
maskP0P15               long    %00000000_00000000_11111111_11111111 ' for masking words
maskP16P20              long    %00000000_00011111_00000000_00000000
maskP16P27              long    %00001111_11111111_00000000_00000000
                        fit     496      